# This file can be modified. If file exists it wont be replaced by "szndaogen" any more.
# Automatically generated Manager class
# Generated by "szndaogen" tool

from .base.{{ tableName }}_manager_base import {{modelName}}ManagerBase


class {{modelName}}Manager({{modelName}}ManagerBase):
    pass

